################ Copyright 2005-2017 Team GoldenEye: Source #################
#
# This file is part of GoldenEye: Source's Python Library.
#
# GoldenEye: Source's Python Library is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the License,
# or(at your option) any later version.
#
# GoldenEye: Source's Python Library is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GoldenEye: Source's Python Library.
# If not, see <http://www.gnu.org/licenses/>.
#############################################################################
from . import GEScenario
from .Utils.GEWarmUp import GEWarmUp
from .Utils import OppositeTeam, _
from .Utils.GEPlayerTracker import GEPlayerTracker
from .Utils import GetPlayers, clamp, _
import GEUtil, GEMPGameRules as GERules, GEGlobal as Glb, GEPlayer, GEWeapon

# TeamArsenal
# Converted for teams by Null
# Originally coded by Troy, Killer Monkey, and E-S
# /////////////////////////// Scenario Data ///////////////////////////

USING_API = Glb.API_VERSION_1_2_0
maxLevel = 8

class TeamArsenal(GEScenario):
    def __init__(self):
        GEScenario.__init__(self)
        self.WaitingForPlayers = True
        self.weaponList = []
        self.warmupTimer = GEWarmUp(self)
        # CVar Holders

        self.TEAM_SCORES = {
            Glb.TEAM_JANUS: {
                'name': "JANUS",
                'Levels': 0,
                'Kills': 0
            },
            Glb.TEAM_MI6: {
                'name': "MI6",
                'Levels': 0,
                'Kills': 0
            }
        }

    def GetPrintName(self):
        return "Team Arsenal"

    def GetScenarioHelp(self, help_obj):
        help_obj.SetDescription("#GES_GP_ARSENAL_HELP")

    def GetGameDescription(self):
        return "Team Arsenal"

    def GetTeamPlay(self):
        return Glb.TEAMPLAY_ALWAYS

    def OnLoadGamePlay(self):
        GEUtil.PrecacheSound("GEGamePlay.Token_Drop_Enemy")  # Used for final weapon.
        GEUtil.PrecacheSound("GEGamePlay.Level_Up")  # Plays when level is gained
        GEUtil.PrecacheSound("GEGamePlay.Level_Down")  # Plays when level is lost
        self.CreateCVar("ar_warmuptime", "15", "The warmup time in seconds. (Use 0 to disable)")

        # Make sure we don't start out in wait time or have a warmup if we changed gameplay mid-match
        if GERules.GetNumActivePlayers() >= 2:
            self.WaitingForPlayers = False
            self.warmupTimer.StartWarmup(0)

        GERules.EnableSuperfluousAreas()
        GERules.EnableInfiniteAmmo()
        GERules.SetAllowTeamSpawns(True)
        GERules.SetSpawnInvulnTime(1, False)

    def OnUnloadGamePlay(self):
        super(TeamArsenal, self).OnUnloadGamePlay()
        self.warmupTimer = None

    def OnCVarChanged(self, name, oldvalue, newvalue):
        if name == "ar_warmuptime":
            if self.warmupTimer.IsInWarmup():
                val = int(newvalue)
                self.warmupTimer.StartWarmup(val)
                if val <= 0:
                    GERules.EndRound(False)

    def OnRoundBegin(self):
        GERules.AllowRoundTimer(False)

        #Remove all pickups
        GERules.DisableWeaponSpawns()
        GERules.DisableAmmoSpawns()
        GERules.DisableArmorSpawns()

        # Clear our weapon list
        self.weaponList = []

        # Store all the current weaponset's weapons in a list for easy access.
        for i in range(0, 8):
            self.weaponList.append(GEWeapon.WeaponClassname(GERules.GetWeaponInSlot(i)))

        #Track team levels and kills
        self.TEAM_SCORES = {
            Glb.TEAM_JANUS: {
                'name': "JANUS",
                'Levels': 0,
                'Kills': 0
            },
            Glb.TEAM_MI6: {
                'name': "MI6",
                'Levels': 0,
                'Kills': 0
            }
        }

        # Reset both team scores and all players kills and deaths
        GERules.GetTeam(Glb.TEAM_MI6).SetRoundScore(1)
        GERules.GetTeam(Glb.TEAM_JANUS).SetRoundScore(1)
        GERules.ResetAllPlayersScores()

    def OnPlayerSpawn(self, player):
        if not self.WaitingForPlayers and not self.warmupTimer.IsInWarmup():
            if player.IsInitialSpawn():
                GEUtil.PopupMessage(player, "Team Arsenal", "Your entire team is given a new \n weapon per team kills until the last weapon to win!")  # GES_GPH_AR_GOAL
            else:
                self.ar_GivePlayerWeapons(player)
        elif GERules.GetNumInRoundPlayers() < 2:
            #Make sure teams have levels for warmup
            GERules.GetTeam(Glb.TEAM_MI6).SetRoundScore(1)
            GERules.GetTeam(Glb.TEAM_JANUS).SetRoundScore(1)


    def OnPlayerKilled(self, victim, killer, weapon):
        if self.WaitingForPlayers or self.warmupTimer.IsInWarmup() or GERules.IsIntermission() or not victim:
            return
        vL = self.TEAM_SCORES[victim.GetTeamNumber()]['Levels']
        # Convert projectile entity names to their corresponding weapon
        name = weapon.GetClassname().lower()
        if name.startswith("npc_"):
            if name == "npc_grenade":
                name = "weapon_grenade"
            elif name == "npc_rocket":
                name = "weapon_rocket_launcher"
            elif name == "npc_shell":
                name = "weapon_grenade_launcher"
            elif name == "npc_mine_remote":
                name = "weapon_remotemine"
            elif name == "npc_mine_timed":
                name = "weapon_timedmine"
            elif name == "npc_mine_proximity":
                name = "weapon_proximitymine"

        if not killer or victim == killer:  # World kill or suicide
            self.ar_IncrementKills(victim, -1)
        else:
            # Normal kill
            if name == "weapon_slappers" or name == "player":  # Slappers or killbind.
                pass
            if maxLevel == self.ar_GetLevel(killer):
                self.ar_IncrementKills(killer, 1)
            else:
                self.ar_IncrementKills(killer, 1)
        victim.StripAllWeapons()  # This prevents the victim from dropping weapons, which might confuse players since there are no pickups.

    # Advance the given player's level kills by the given amount.
    def ar_IncrementKills(self, player, amt):
        if amt > 0 or amt < 0 and player.GetScore() > 0:
            self.ar_SetKills(player, self.TEAM_SCORES[player.GetTeamNumber()]['Kills'] + amt)
        player.SetScore(player.GetScore() + amt)

    # Advance the given player's level by the given amount.
    def ar_IncrementLevel(self, player, amt):
        if self.TEAM_SCORES[player.GetTeamNumber()]['Levels'] + amt > 0:  # Make sure there's enough levels to increment
            self.ar_SetLevel(player, self.TEAM_SCORES[player.GetTeamNumber()]['Levels'] + amt)

    def ar_SetKills(self, player, kills):
        if not player:
            return
        if kills >= 1 + GERules.GetNumInRoundTeamPlayers(player.GetTeamNumber()) // 2:
            self.ar_IncrementLevel(player, 1)
            self.TEAM_SCORES[player.GetTeamNumber()]['Kills'] = 0
        elif kills < 0:
            self.ar_IncrementLevel(player, -1)
            self.TEAM_SCORES[player.GetTeamNumber()]['Kills'] = 1 + GERules.GetNumInRoundTeamPlayers(player.GetTeamNumber()) // 2 - 1
        else:
            self.TEAM_SCORES[player.GetTeamNumber()]['Kills'] = kills
            for teamplayer in GetPlayers():
                if (teamplayer.GetTeamNumber() == player.GetTeamNumber()):
                    self.ar_PrintCurKills(teamplayer)

    # Set the given player's level to the given amount and give them their weapon.
    def ar_SetLevel(self, player, lvl):
        if not player:
            return

        oldlvl = (self.TEAM_SCORES[player.GetTeamNumber()]['Levels'])
        self.TEAM_SCORES[player.GetTeamNumber()]['Levels'] = lvl
        GERules.GetTeam(player.GetTeamNumber()).SetRoundScore(lvl + 1)  # Correcting for the fact that "self.TEAM_SCORES[player.GetTeamNumber()]['Levels']" started at 0
        GEUtil.EmitGameplayEvent("ar_levelchange", str(player.GetUserID()), str(lvl))  #Not sure what this is for, so leaving it
        if lvl != oldlvl and lvl <= maxLevel:
            if self.ar_GetLevel(player) == maxLevel:  # Final level gets a special sound and announces to everyone.
                msg = _("#GES_GP_AR_FINALWEAPON", "^y" + player.GetCleanPlayerName() + "^l on Team ^y" + str(self.TEAM_SCORES[player.GetTeamNumber()]['name']))
                GEUtil.PostDeathMessage(msg)
                for allplayers in GetPlayers():
                    GEUtil.PlaySoundTo(allplayers, "GEGamePlay.Token_Grab_Enemy")
            elif lvl > oldlvl:  # Gained a level
                for teamplayer in GetPlayers():
                    if teamplayer.GetTeamNumber() == player.GetTeamNumber():
                        GEUtil.PlaySoundTo(teamplayer, "GEGamePlay.Level_Up")
            else:  # Lost a level.
                for teamplayer in GetPlayers():
                    if teamplayer.GetTeamNumber() == player.GetTeamNumber():
                        GEUtil.PlaySoundTo(teamplayer, "GEGamePlay.Level_Down")

        # Give weapons and print level.
        if lvl <= maxLevel:  # give weapons if current lvl is under max level.
            for teamplayer in GetPlayers():
                if teamplayer.GetTeamNumber() == player.GetTeamNumber():
                    self.ar_PrintCurTeamLevelToPlayer(teamplayer)
                    self.ar_GivePlayerWeapons(teamplayer)
        elif not GERules.IsIntermission():  # In fact, if we are, we just won!
            #GEUtil.EmitGameplayEvent("ar_completedarsenal", str(allplayers.GetUserID()), "", "", "", True)  #Not sure what this is for, so leaving it out
            GERules.EndRound()

    def OnThink(self):
        # Check to see if we can get out of warmup
        if self.WaitingForPlayers and GERules.GetNumActivePlayers() > 1:
            self.WaitingForPlayers = False
            if not self.warmupTimer.HadWarmup():
                self.warmupTimer.StartWarmup(int(GEUtil.GetCVarValue("ar_warmuptime")), True)
            else:
                GERules.EndRound(False)

    def CanPlayerHaveItem(self, player, item):
        if not player or player.GetTeamNumber() == Glb.TEAM_SPECTATOR or player.GetTeamNumber() == Glb.TEAM_NONE:
            return

        weapon = GEWeapon.ToGEWeapon(item)
        if weapon:
            name = weapon.GetClassname().lower()
            tL = self.TEAM_SCORES[player.GetTeamNumber()]['Levels']
            if name == "weapon_slappers":
                return True
            if len(self.weaponList) < tL:
                return True
            if len(self.weaponList) == tL:
                return name == "weapon_knife"
            if name == self.weaponList[int(tL)]:
                return True
            return False

    def CanMatchEnd(self):
        #We just finished a round or it's not possible to get kills.
        if GERules.IsIntermission() or GERules.GetNumActivePlayers() < 2:
            return True
        else:
            return False

    # /////////////////////////// Utility Functions ///////////////////////////
    # Give player their level specific weapon and the slappers
    def ar_GivePlayerWeapons(self, player):
        if not player or player.IsDead() or len(self.weaponList) < 7:
            return
        player.StripAllWeapons()
        player.GiveNamedWeapon("weapon_slappers", 0)  # We only ever have slappers and the weapon of our level
        lvl = int(self.TEAM_SCORES[player.GetTeamNumber()]['Levels'])
        if maxLevel > lvl:
            weap = self.weaponList[lvl]
        else:
            weap = "weapon_knife"  # Last level is always the knife.
        if weap != "weapon_slappers":
            player.GiveNamedWeapon(weap, 800)  # We use infinite ammo so who cares about exact ammo amounts
        player.WeaponSwitch(weap)

    # Present a HUD message with the player's team current kills
    def ar_PrintCurKills(self, player):
        if not player:
            return
        msg = _("#GES_GP_GUNGAME_KILLS", str(1 + GERules.GetNumInRoundTeamPlayers(player.GetTeamNumber()) // 2 - int(self.TEAM_SCORES[player.GetTeamNumber()]['Kills'])))
        GEUtil.HudMessage(player, msg, -1, 0.71, GEUtil.Color(220, 220, 220, 255), 1.5, 2)

    # Present a HUD message with the player's team current level
    def ar_PrintCurTeamLevelToPlayer(self, player):
        if not player:
            return
        lvl = int(self.TEAM_SCORES[player.GetTeamNumber()]['Levels'])
        if lvl < maxLevel:
            name = GEWeapon.WeaponPrintName(self.weaponList[lvl])
        else:
            name = "Hunting Knife"
        msg = _("#GES_GP_GUNGAME_LEVEL", str(lvl + 1), name)
        GEUtil.HudMessage(player, msg, -1, 0.71, GEUtil.Color(220, 220, 220, 255), 3.0, 2)

    # Returns the given player's level
    def ar_GetLevel(self, player):
        if not player:
            return -1
        return self.TEAM_SCORES[player.GetTeamNumber()]['Levels']
